package john111898.ld29;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.JFrame;

import john111898.ld29.control.KeyboardHandler;
import john111898.ld29.control.MouseHandler;
import john111898.ld29.entity.EntityPlayer;
import john111898.ld29.image.ImageLoader;
import john111898.ld29.level.Level;
import john111898.ld29.level.LevelGenerator;
import john111898.ld29.render.RenderComponent;
import john111898.ld29.sound.Sound;
import john111898.ld29.sound.SoundManager;
import john111898.ld29.world.Ground;
import john111898.ld29.world.PitRender;

public class Game {
	static JFrame window;
	public static final int WIDTH = 640;
	public static final int HEIGHT = 480;
	static final String title = "Into the Earth";
	
	public static RenderComponent render;
	static RenderThread rThread;
	
	static GameThread gThread;
	
	public static BufferedImage[] images = new BufferedImage[64];
	
	public static EntityPlayer player;
	public static Ground levelGround;
	public static Level currentLevel;
	
	static int levelNumber = 0;
	
	static SoundManager sounds;
	static ArrayList<Sound> soundList = new ArrayList<Sound>();
	
	public static void init() {
		window = new JFrame(title);
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setSize(WIDTH, HEIGHT);
		window.setResizable(false);
		window.setVisible(true);
		render = new RenderComponent();
		window.add(render);
		window.addKeyListener(new KeyboardHandler());
		window.addMouseListener(new MouseHandler());
		window.addMouseMotionListener(new MouseHandler());
		
		images[0] = ImageLoader.loadImage("player");
		images[1] = ImageLoader.loadImage("background1");
		images[2] = ImageLoader.loadImage("ground1");
		images[3] = ImageLoader.loadImage("button1");
		images[4] = ImageLoader.loadImage("box");
		images[5] = ImageLoader.loadImage("button2");
		images[6] = ImageLoader.loadImage("pit");
		images[7] = ImageLoader.loadImage("background2");
		images[8] = ImageLoader.loadImage("ground2");
		images[9] = ImageLoader.loadImage("counterRed");
		images[10] = ImageLoader.loadImage("counterGreen");
		images[11] = ImageLoader.loadImage("backgroundT");
		images[12] = ImageLoader.loadImage("groundT");
		images[13] = ImageLoader.loadImage("block1");
		images[14] = ImageLoader.loadImage("block2");
		images[15] = ImageLoader.loadImage("vacuumUpTemp");
		images[16] = ImageLoader.loadImage("vacuumDownTemp");
		images[17] = ImageLoader.loadImage("vacuumLeftTemp");
		images[18] = ImageLoader.loadImage("vacuumRightTemp");
		images[19] = ImageLoader.loadImage("background3");
		images[20] = ImageLoader.loadImage("ground3");
		images[21] = ImageLoader.loadImage("block3");
		images[22] = ImageLoader.loadImage("background4");
		images[23] = ImageLoader.loadImage("ground4");
		images[24] = ImageLoader.loadImage("block4");
		images[25] = ImageLoader.loadImage("background5");
		images[26] = ImageLoader.loadImage("ground5");
		images[27] = ImageLoader.loadImage("background6");
		images[28] = ImageLoader.loadImage("ground6");
		images[29] = ImageLoader.loadImage("background7");
		images[30] = ImageLoader.loadImage("ground7");
		images[31] = ImageLoader.loadImage("block5");
		images[32] = ImageLoader.loadImage("block6");
		images[33] = ImageLoader.loadImage("radius100");
		images[34] = ImageLoader.loadImage("radius150");
		images[35] = ImageLoader.loadImage("radius300");
		images[36] = ImageLoader.loadImage("blank");
		
		window.setIconImage(images[4]);
		
		sounds = new SoundManager();
		soundList.add(new Sound("activate.wav"));
		soundList.add(new Sound("pickup.wav"));
		soundList.add(new Sound("drop.wav"));
		soundList.add(new Sound("pit.wav"));
		soundList.add(new Sound("button1.wav"));
		soundList.add(new Sound("button2.wav"));
		soundList.add(new Sound("button3.wav"));
		soundList.add(new Sound("vacuumRotate.wav"));
		
		currentLevel = LevelGenerator.tutorial();
		levelGround = currentLevel.ground;
		
		player = new EntityPlayer(images[0], 250, 150);
		player.setX(WIDTH / 2);
		player.setY(HEIGHT - 200);
		currentLevel.addRenderable(player);
		currentLevel.addRenderable(new PitRender());
		
		render.setLevelNumber(currentLevel.getLevelNumber());
		render.resetRenderNumber();
		render.repaint();
		
		gThread = new GameThread();
		gThread.start();
		
		rThread = new RenderThread();
		rThread.start();
	}
	
	public static void addImageToRender(BufferedImage image, int x, int y) {
		render.addImage(image, x, y);
	}
	
	public static void setPlayerX(int x) {
		player.setX(x);
	}
	
	public static void setPlayerY(int y) {
		player.setY(y);
	}
	
	public static void updateRender() {
		currentLevel.addNewRenderables();
		render.resetRenderNumber();
		render.repaint();
	}
	
	public static void updateLogic() {
		int groundWidth = levelGround.getImage().getWidth();
		
		if (player.getLogicX() < WIDTH / 2) {
			player.setRenderX(player.getLogicX());
		}
		else {
			if (player.getLogicX() > groundWidth - WIDTH) {
				player.setRenderX(player.getLogicX() - (groundWidth - WIDTH) + WIDTH / 2);
			}
			else {
				player.setRenderX(Game.WIDTH / 2);
			}
		}
		
		if (player.renderX == WIDTH / 2) {
			levelGround.setAdjustedX(-player.getLogicX());
			currentLevel.background.x = -player.getLogicX();
		}
		
		if (player.renderX < 0) {
			player.renderX = 0;
			player.setX(0);
		}
		
		if (player.renderX > WIDTH) {
			player.renderX = WIDTH - 8;
			player.setX(groundWidth - WIDTH / 2);
		}
		
		if (player.getHorizontalVelocityPositive() > 10.0) {
			player.setHorizontalVelocityPositive(10.0);
		}
		if (player.getHorizontalVelocityNegative() > 10.0) {
			player.setHorizontalVelocityNegative(10.0);
		}
		if (player.getHorizontalVelocityNegative() > 1.5) {
			player.setHorizontalVelocityNegative(player.getHorizontalVelocityNegative() - 1.5);
		}
		else {
			if (player.getHorizontalVelocityNegative() > 0.0 && player.getHorizontalVelocityNegative() <= 1.5) {
				player.setHorizontalVelocityNegative(0.0);
			}
		}
		
		if (player.getHorizontalVelocityPositive() > 1.5) {
			player.setHorizontalVelocityPositive(player.getHorizontalVelocityPositive() - 1.5);
		}
		else {
			if (player.getHorizontalVelocityPositive() > 0.0 && player.getHorizontalVelocityPositive() <= 1.5) {
				player.setHorizontalVelocityPositive(0.0);
			}
		}
		player.updateX();
		
		currentLevel.updateGameObjects(player);
		currentLevel.doCollisions();
		currentLevel.checkForPitOpen();
	}
	
	public static void passMouseClickToCurrentLevel(int x, int y, boolean leftClick) {
		currentLevel.mouseClick(x, y, leftClick);
	}
	
	public static void nextLevel() {
		playSound(3);
		switch (currentLevel.getLevelNumber()) {
		case 0:
			currentLevel = LevelGenerator.levelOne();
			break;
		case 1:
			currentLevel = LevelGenerator.levelTwo();
			break;
		case 2:
			currentLevel = LevelGenerator.levelThree();
			break;
		case 3:
			currentLevel = LevelGenerator.levelFour();
			break;
		case 4:
			currentLevel = LevelGenerator.levelFive();
			break;
		case 5:
			currentLevel = LevelGenerator.levelSix();
			break;
		case 6:
			currentLevel = LevelGenerator.levelSeven();
			break;
		case 7:
			currentLevel = LevelGenerator.tutorial();
			break;
		}
		levelGround = currentLevel.ground;
		
		player = new EntityPlayer(images[0], 250, 150);
		player.setX(WIDTH / 2);
		player.setY(HEIGHT - 200);
		
		currentLevel.addRenderable(player);
		currentLevel.addRenderable(new PitRender());
		render.setLevelNumber(currentLevel.getLevelNumber());
		render.resetRenderNumber();
		render.resetImages();
		render.resetSpecialText();
		render.repaint();
	}
	
	public static void addSpecialTextToRender(int x, int y, String t, Color c) {
		render.addSpecialTextToRender(x, y, t, c);
	}
	
	public static boolean checkPlayerJump() {
		int height = currentLevel.playerCollide(player);
		return (Game.player.getY() < height - Game.player.getHeight() - 16);
	}
	
	public static void playSound(int id) {
		sounds.playSound(soundList.get(id));
	}
	
	public static void restartLevel() {
		switch (currentLevel.getLevelNumber()) {
		case 0:
			currentLevel = LevelGenerator.tutorial();
			break;
		case 1:
			currentLevel = LevelGenerator.levelOne();
			break;
		case 2:
			currentLevel = LevelGenerator.levelTwo();
			break;
		case 3:
			currentLevel = LevelGenerator.levelThree();
			break;
		case 4:
			currentLevel = LevelGenerator.levelFour();
			break;
		case 5:
			currentLevel = LevelGenerator.levelFive();
			break;
		case 6:
			currentLevel = LevelGenerator.levelSix();
			break;
		case 7:
			currentLevel = LevelGenerator.levelSeven();
			break;
		}
		
		levelGround = currentLevel.ground;
		
		player = new EntityPlayer(images[0], 250, 150);
		player.setX(WIDTH / 2);
		player.setY(HEIGHT - 200);
		
		currentLevel.addRenderable(player);
		currentLevel.addRenderable(new PitRender());
		render.setLevelNumber(currentLevel.getLevelNumber());
		render.resetRenderNumber();
		render.resetImages();
		render.resetSpecialText();
		render.repaint();
	}
}