package john111898.ld29.level;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

import john111898.ld29.Game;
import john111898.ld29.entity.EntityPlayer;
import john111898.ld29.render.DefaultRenderable;
import john111898.ld29.render.Renderable;
import john111898.ld29.world.ClickableButton;
import john111898.ld29.world.GameObject;
import john111898.ld29.world.Ground;
import john111898.ld29.world.ObjectCounter;
import john111898.ld29.world.Vacuum;
import john111898.ld29.world.WorldBlock;

public class Level {
	ArrayList<Renderable> renderables = new ArrayList<Renderable>();
	int levelNumber;
	
	public DefaultRenderable background;
	public Ground ground;
	
	Random r = new Random();
	
	public long elevation;
	
	public int playerRadius = 300;
	
	public void addNewRenderables() {
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			Game.addImageToRender(r.getImage(), r.getX(), r.getY());
		}
	}
	
	public Level(int levelNumber, long elevation) {
		this.levelNumber = levelNumber;
		this.elevation = elevation;
	}
	
	public void addRenderable(Renderable r) {
		renderables.add(r);
	}
	
	public void mouseClick(int x, int y, boolean leftClick) {
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			if (r instanceof ClickableButton) {
				ClickableButton cb = (ClickableButton) r;
				if (x > cb.getX() && x < cb.x2 && y > cb.getY() && y < cb.y2) {
					cb.click();
					switch(this.r.nextInt(2)) {
					case 0:
						Game.playSound(4);
						break;
					case 1:
						Game.playSound(5);
						break;
					case 2:
						Game.playSound(6);
						break;
					}
				}
			}
			else {
				if (r instanceof GameObject) {
					GameObject go = (GameObject) r;
					if ((x > go.getX() && x < go.x2 && y > go.getY() && y < go.y2) && go.canBePickedUp && (Math.sqrt(Math.pow((go.getX() - Game.player.getX()), 2) + (Math.pow(go.getY() - Game.player.getY(), 2))) < 300)) {
						go.setHeldByPlayer(!go.isHeldByPlayer());
						if (go.isHeldByPlayer()) {
							Game.playSound(1);
						}
						else {
							Game.playSound(2);
						}
					}
					else {
						if (go.isHeldByPlayer() && !leftClick) {
							if (Math.sqrt(Math.pow((go.getX() - x), 2) + (Math.pow(go.getY() - y, 2))) < 300) {
								go.setHeldByPlayer(false);
								go.setLogicX(x - (Game.WIDTH / 2) + go.getLogicX() - 24);
								go.updateRenderX();
								go.setY(y - 32);
								Game.playSound(2);
							}
						}
					}
					if ((x > go.getX() && x < go.x2 && y > go.getY() && y < go.y2) && go instanceof Vacuum) {
						Vacuum v = (Vacuum) r;
						v.rotateVacuum();
					}
				}
			}
		}
	}
	
	public void updateGameObjects(EntityPlayer player) {
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			if (r instanceof GameObject) {
				GameObject go = (GameObject) r;
				if (go.isHeldByPlayer()) {
					go.setLogicX(player.getLogicX() + 8);
					go.updateRenderX();
					go.setY(player.getY() - 16);
				}
				go.updateRenderX();
				
				if (!go.isHeldByPlayer() && go.affectedByGravity()) {
					if (go.getY() < verticalCollide(go) - go.getImageHeight()) {
						go.setVerticalVelocity(go.getVerticalVelocity() - 0.25);
					}
					else {
						go.setVerticalVelocity(0.0);
					}
					go.updateHeight();
				}
				
			}
			if (r instanceof ObjectCounter) {
				ObjectCounter oc = (ObjectCounter) r;
				oc.countObjects();
			}
			if (r instanceof Vacuum) {
				Vacuum v = (Vacuum) r;
				v.attractObjects();
			}
		}
	}
	
	public int getLevelNumber() {
		return levelNumber;
	}
	
	// returns the height where a collision would/is happen(ing), could be ground if no objects
	public int verticalCollide(GameObject o) {
		ArrayList<GameObject> check = getGameObjects();
		check.remove(o);
		Collections.reverse(check);
		o.updateCollisionCoords(o.getX(), o.getY());
		for (int i = 0; i < check.size(); i++) {
			GameObject test = check.get(i);
			test.updateCollisionCoords(test.getX(), test.getY());
			if (test.getCollisionBox().intersects(o.getCollisionBox())) {
				if (o.getY() < test.getY()) {
					return test.getY();
				}
			}
		}
		return Game.HEIGHT - Game.levelGround.getGroundHeight();
	}
	
	public int playerCollide(EntityPlayer p) {
		ArrayList<GameObject> check = getGameObjects();
		p.updateCollisionCoords(p.getX() + 8, p.getY());
		for (int i = 0; i < check.size(); i++) {
			GameObject test = check.get(i);
			test.updateCollisionCoords(test.getX(), test.getY());
			if (test.getCollisionBox().intersects(p.getCollisionBox())) {
				if (p.getY() < test.getY()) {
					return test.getY();
				}
			}
		}
		return Game.HEIGHT - Game.levelGround.getGroundHeight();
	}
	
	public int playerCollideSide(EntityPlayer p) {
		ArrayList<WorldBlock> check = getWorldBlocks();
		p.updateCollisionCoords(p.getX() + 8, p.getY());
		for (int i = 0; i < check.size(); i++) {
			GameObject test = check.get(i);
			test.updateCollisionCoords(test.getX(), test.getY());
			if (test.getCollisionBox().intersects(p.getCollisionBox())) {
				if (p.getX() < test.getX()) {
					return test.getX();
				}
				else {
					if (p.getX() > test.getX()) {
						return p.getX();
					}
				}
			}
		}
		return 0;
	}
	
	public int objectCollideSide(GameObject o) {
		ArrayList<WorldBlock> check = getWorldBlocks();
		o.updateCollisionCoords(o.getX() + 32, o.getY());
		for (int i = 0; i < check.size(); i++) {
			GameObject test = check.get(i);
			test.updateCollisionCoords(test.getX(), test.getY());
			if (test.getCollisionBox().intersects(o.getCollisionBox())) {
				if (o.getX() < test.getX()) {
					return test.getX();
				}
				else {
					if (o.getX() > test.getX()) {
						return o.getX();
					}
				}
			}
		}
		return 0;
	}
	
	public void doCollisions() {
		if (Game.player.getY() + Game.player.getHeight() >= playerCollide(Game.player) && !(Game.player.getLogicX() > Game.levelGround.getPitX() && Game.player.getLogicX() < Game.levelGround.getPitX() + 16 && Game.levelGround.isPitOpen())) {
			Game.player.setY(playerCollide(Game.player) - Game.player.getHeight());
			Game.player.setVerticalVelocity(0);
		}
		
		if (Game.player.getY() < playerCollide(Game.player) - Game.player.getHeight()) {
			Game.player.setVerticalVelocity(Game.player.getVerticalVelocity() - 0.25);
		}
		
		if (Game.player.getLogicX() > Game.levelGround.getPitX() && Game.player.getLogicX() < Game.levelGround.getPitX() + 16 && Game.levelGround.isPitOpen()) {
			Game.player.setVerticalVelocity(Game.player.getVerticalVelocity() - 0.25);
			Game.nextLevel();
		}
		
		if (playerCollideSide(Game.player) == Game.player.getX()) {
			Game.player.setHorizontalVelocityPositive(0.0);
			Game.player.setHorizontalVelocityNegative(2.0);
		}
		else {
			if (playerCollideSide(Game.player) != 0) {
				Game.player.setHorizontalVelocityPositive(2.0);
				Game.player.setHorizontalVelocityNegative(0.0);
			}
		}
		
		Game.player.updateHeight();
	}
	
	public void setBackground(DefaultRenderable bg) {
		background = bg;
	}
	
	public void setGround(Ground ground) {
		this.ground = ground;
	}
	
	public ArrayList<GameObject> getGameObjectsMinusCounter() {
		ArrayList<GameObject> go = new ArrayList<GameObject>();
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			if (r instanceof GameObject && !(r instanceof ObjectCounter)) {
				go.add((GameObject) r);
			}
		}
		return go;
	}
	
	public ArrayList<GameObject> getGameObjects() {
		ArrayList<GameObject> go = new ArrayList<GameObject>();
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			if (r instanceof GameObject) {
				go.add((GameObject) r);
			}
		}
		return go;
	}
	
	public ArrayList<WorldBlock> getWorldBlocks() {
		ArrayList<WorldBlock> wb = new ArrayList<WorldBlock>();
		for (int i = 0; i < renderables.size(); i++) {
			Renderable r = renderables.get(i);
			if (r instanceof WorldBlock) {
				wb.add((WorldBlock) r);
			}
		}
		return wb;
	}
	
	public void checkForPitOpen() {
		
	}
}