package john111898.ld29.world;

import java.awt.image.BufferedImage;
import java.util.ArrayList;

import john111898.ld29.Game;

public class Vacuum extends GameObject {
	int direction = 0;
	public static final int DIRECTION_LEFT = 0;
	public static final int DIRECTION_RIGHT = 1;
	public static final int DIRECTION_UP = 2;
	public static final int DIRECTION_DOWN = 3;
	boolean canRotate = true;
	
	public Vacuum(int x, int y, BufferedImage image, int direction) {
		super(x, y, image, false);
		this.direction = direction;
	}
	
	public Vacuum(int x, int y, BufferedImage image, int direction, boolean pickUp) {
		super(x, y, image, pickUp);
		this.direction = direction;
	}
	
	public Vacuum(int x, int y, BufferedImage image, int direction, boolean pickUp, boolean canRotate) {
		super(x, y, image, pickUp);
		this.direction = direction;
		this.canRotate = canRotate;
	}
	
	public void attractObjects() {
		ArrayList<GameObject> objects = Game.currentLevel.getGameObjects();
		objects.remove(this);
		for (int i = 0; i < objects.size(); i++) {
			GameObject o = objects.get(i);
			if (!(o instanceof Vacuum) && o.affectedByGravity) {
				switch (direction) {
				case DIRECTION_LEFT:
					o.setLogicX(o.getLogicX() - isObjectInRange(o));
					break;
				case DIRECTION_RIGHT:
					o.setLogicX(o.getLogicX() + isObjectInRange(o));
					break;
				case DIRECTION_UP:
					o.setVerticalVelocity(o.getVerticalVelocity() + isObjectInRange(o));
					break;
				case DIRECTION_DOWN:
					o.setVerticalVelocity(o.getVerticalVelocity() - isObjectInRange(o));
					break;
				}
			}
		}
	}
	
	public int isObjectInRange(GameObject o) {
		switch (direction) {
		case DIRECTION_LEFT:
			if (((o.y > this.y && o.y2 < this.y2) || (o.y < this.y && o.y2 > this.y) || (o.y < this.y2 && o.y > this.y)) && (o.x > this.x)) {
				return (int) (1000 / (this.x - o.x));
			}
			else {
				return 0;
			}
		case DIRECTION_RIGHT:
			if (((o.y > this.y && o.y2 < this.y2) || (o.y < this.y && o.y2 > this.y) || (o.y < this.y2 && o.y > this.y)) && (o.x < this.x)) {
				return (int) (1000 / (o.x - this.x));
			}
			else {
				return 0;
			}
		case DIRECTION_UP:
			if (((o.x > this.x && o.x2 < this.x2) || (o.x < this.x && o.x2 > this.x) || (o.x < this.x2 && o.x > this.x)) && (o.y > this.y)) {
				return (int) (200 / (this.y - o.y));
			}
			else {
				return 0;
			}
		case DIRECTION_DOWN:
			if (((o.x > this.x && o.x2 < this.x2) || (o.x < this.x && o.x2 > this.x) || (o.x < this.x2 && o.x > this.x)) && (o.y < this.y)) {
				return (int) (200 / (o.y - this.y));
			}
			else {
				return 0;
			}
		}
		return 0;
	}
	
	public void rotateVacuum() {
		if (canRotate) {
			Game.playSound(7);
			switch (direction) {
			case DIRECTION_LEFT:
				direction = DIRECTION_UP;
				image = Game.images[15];
				break;
			case DIRECTION_RIGHT:
				direction = DIRECTION_DOWN;
				image = Game.images[16];
				break;
			case DIRECTION_UP:
				direction = DIRECTION_RIGHT;
				image = Game.images[18];
				break;
			case DIRECTION_DOWN:
				direction = DIRECTION_LEFT;
				image = Game.images[17];
				break;
			}
		}
	}
}